@@ -15,6 +15,18 @@ DOC "class Base" {
  METHODS
 	Base provides the following generic methods to all objects:
 
+	new		- EXPERIMENTAL! Arranges to create a new object of
+			  the class of the invoking object.
+
+	instance	- EXPERIMENTAL! Arranges to create a new object of
+			  the class of the invoking object.  This method
+			  differs from `new' by NOT automatically invoking
+			  the `init' method of the new object.
+			  One possible usage:  Create a replacement for the
+			  normal class object -a replacement which has no
+			  hard-coded methods (this will need careful design
+			  though).
+
 	init		- Does nothing.  The init method is automatically
 			  invoked whenever an object is created with `new'.
 
@@ -80,9 +92,11 @@ DOC "class Base" {
 	instvars	- (I) Returns the names of all existing instance variables.
 	iclassvars	- (I) List instance class variables
 
-	classvars	- (C) List class variables
-	objects		- (C) List objects of this class
-	methods		- (C) List methods defined in this class
+	classvars	- (C) List class variables.
+	objects		- (C) List objects of this class.
+	methods		- (C) List methods defined in this class.
+	sysmethods	- (C) List system methods defined in this class.
+	cached		- (C) List cached methods for this class.
 	body <method>	- (C) List the body of a method.
 	args <method>	- (C) List formal parameters for a method.
 
@@ -213,41 +227,44 @@ crunch_skip end
 
 class Base
 
-Base method init { args } {}
+Base method init args {}
 
-Base method destroy { args } {
-	freeObj $self
+Base method destroy args {
+	otFreeObj $self
 }
 
-Base method class { args } {
+Base method class args {
 	return $iclass
 }
 
 # Note: The `set' method takes on the class of the caller, so
 #	instvars will use the callers scope.
 #
-Base method set { args } {
+Base method set args {
 	set class $iclass
-	instvar [lindex $args 0]
+	# instvar [lindex $args 0]
+	set var [lindex $args 0]
+	regexp -- {^([^(]*)\(.*\)$} $var m var
+	instvar $var
 	return [eval set $args]
 }
 
-Base method eval { l } {
+Base method eval l {
 	return [eval $l]
 }
 
 Base method info { cmd args } {
 	switch $cmd {
-	"instvars"	{return [eval {objinfoVars\
+	"instvars"	{return [eval {otObjInfoVars\
 		_oIV_${iclass}:${self}: _oIV_${iclass}:${self}:} $args]}
-	"iclassvars"	{objinfoVars _oICV_${iclass}: _oICV_${iclass}: $args}
-	"classvars"	{objinfoVars _oDCV_${iclass}: _oDCV_${iclass}: $args}
-	"objects"	{objinfoObjects $iclass}
-	"methods"	{classInfoMethods $iclass}
-	"sysmethods"	{classInfoSysMethods $iclass}
-	"cached"	{classInfoCached $iclass}
-	"body"		{classInfoBody $iclass $args}
-	"args"		{classInfoArgs $iclass $args}
+	"iclassvars"	{otObjInfoVars _oICV_${iclass}: _oICV_${iclass}: $args}
+	"classvars"	{otObjInfoVars _oDCV_${iclass}: _oDCV_${iclass}: $args}
+	"objects"	{otObjInfoObjects $iclass}
+	"methods"	{otClassInfoMethods $iclass}
+	"sysmethods"	{otClassInfoSysMethods $iclass}
+	"cached"	{otClassInfoCached $iclass}
+	"body"		{otClassInfoBody $iclass $args}
+	"args"		{otClassInfoArgs $iclass $args}
 
 	"options"	{$iclass::collectOptions values ret
 			return [array get ret] }
@@ -262,12 +279,68 @@ Base method info { cmd args } {
 	}	
 }
 
-Base method unknown { args } {
+Base method unknown args {
 	return -code error \
 		-errorinfo "Undefined method '$method' invoked" \
 		"Undefined method '$method' invoked"
 }
 
+#------- START EXPERIMENTAL
+
+Base method new { obj args } {
+	eval {otNew $iclass $obj} $args
+}
+
+Base method instance { obj args } {
+	eval {otInstance $iclass $obj} $args
+}
+
+Base method sys_method args {
+	eval {otMkMethod S $iclass} $args
+}
+
+Base method method args {
+	eval {otMkMethod N $iclass} $args
+}
+
+Base method del_method args {
+	eval {otRmMethod $iclass} $args
+}
+
+Base method inherit args {
+	eval {otInherit $iclass} $args
+}
+
+# class AnonInst - inherit from this class to be able to generate
+# anonymous objects.  Example:
+#
+#	class Foo
+#	Foo inherit AnonInst
+#	set obj [Foo new]
+#
+# NOTE: EXPERIMENTAL!!!
+
+class AnonInst
+AnonInst method anonPrefix p {
+	iclassvar _prefix
+	set _prefix $p
+}
+AnonInst method new {{obj {}} args} {
+	iclassvar _count _prefix
+	if ![info exists _count] {
+		set _count 0
+	}
+	if ![info exists _prefix] {
+		set _prefix "$iclass"
+	}
+	if ![string compare "" $obj] {
+        	set obj $_prefix[incr _count]
+	}
+	eval next {$obj} $args
+	return $obj
+}
+#------- END EXPERIMENTAL
+
 #----------------------------------------------------------------------
 # Configure stuff
 #----------------------------------------------------------------------
@@ -306,33 +379,33 @@ Base method unknown { args } {
 #   If this fails too, return an error.
 #   Note that the normal call of the method `unknown' is avoided by
 #   telling the unknown handler to avoid this (by means of the global array
-#   "unknBarred").
+#   "_obTcl_unknBarred").
 #
-proc MkSectMethod { class name sect } {
-	$class sys_method $name { args } "
+proc otMkSectMethod { class name sect } {
+	$class sys_method $name args "
 		array set Opts \$args
 		foreach i \[array names Opts\] {
-			global unknBarred
-			set unknBarred(\$class::${sect}:\$i) 1
+			global _obTcl_unknBarred
+			set _obTcl_unknBarred(\$class::${sect}:\$i) 1
 			if \[catch {\$class::$sect:\$i \$Opts(\$i)} err\] {
 				if \[catch {\$class::${sect}_unknown\
 					\$i \$Opts(\$i)}\] {
-					unset unknBarred(\$class::${sect}:\$i)
+					unset _obTcl_unknBarred(\$class::${sect}:\$i)
 				  	error \"Unable to do '$sect \$i \$Opts(\$i)'\n\
 						\t\$err
 					\"
 			 	}
 			}
-			unset unknBarred(\$class::${sect}:\$i)
+			unset _obTcl_unknBarred(\$class::${sect}:\$i)
 		}
 	"
 }
 
 # Note: MkOptHandl is really a part of `option' below.
 #
-proc MkOptHandl {} {
+proc otMkOptHandl {} {
     uplevel 1 {
-	$iclass sys_method "cget" { opt } "
+	$iclass sys_method "cget" opt "
 		classvar classOptions
 		if \[catch {$iclass::cget:\$opt} ret\] {
 			if \[catch {\$class::cget_unknown \$opt} ret\] {
@@ -341,7 +414,7 @@ proc MkOptHandl {} {
 		}
 		return \$ret
 	"
-	MkSectMethod $iclass conf_init init
+	otMkSectMethod $iclass conf_init init
 	$iclass sys_method initialize {} {
 		next
 		classvar optDefaults
@@ -360,15 +433,15 @@ proc MkOptHandl {} {
 		next $mode ret
 		eval instvar [array names optDefaults]
 		foreach i [array names optDefaults] {
-			if { $mode == "defaults" } {
-				set ret(-$i) $optDefaults($i)
-			} else {
+			if [string compare "defaults" $mode] {
 				set ret(-$i) [set $classOptions(-$i)]
+			} else {
+				set ret(-$i) $optDefaults($i)
 			}
 		}
 	}
-	MkSectMethod $iclass conf_verify verify
-	MkSectMethod $iclass configure configure
+	otMkSectMethod $iclass conf_verify verify
+	otMkSectMethod $iclass configure configure
 
 	set _optPriv(section,cget) 1
 	set _optPriv(section,init) 1
@@ -378,10 +451,10 @@ proc MkOptHandl {} {
     }
 }
 
-MkSectMethod Base configure configure
+otMkSectMethod Base configure configure
 
 # _optPriv is used for internal option handling house keeping
-# Note: checking for existence of a proc is not allways a good idea,
+# Note: checking for existence of a proc is not always a good idea,
 # since it may simply be a cached pointer to a inherited method.
 #
 Base method option { opt dflt args } {
@@ -394,12 +467,12 @@ Base method option { opt dflt args } {
 
 	array set tmp $args
 	if ![info exists _optPriv(initialize)] {
-		MkOptHandl
+		otMkOptHandl
 		set _optPriv(initialize) 1
 	}
 	foreach i [array names tmp] {
 		if ![info exists _optPriv(section,$i)] {
-			MkSectMethod $iclass $i $i
+			otMkSectMethod $iclass $i $i
 			set _optPriv(section,$i) 1
 		}
 		$iclass sys_method "$i:$opt" _val "
@@ -613,14 +686,14 @@ class Widget
 #	the instvar `wincmd' will contain the name of the widget's associated
 #	command.
 
-Widget method init { args } {
+Widget method init args {
 	instvar win wincmd
 
 	next
 
 	set first "[lindex $args 0]"
 	set c1 "[string index $first 0]"
-	if { "$c1" == "" || "$c1" == "-" } {
+	if { ![string compare "" "$c1"] || ![string compare "-" "$c1"] } {
 		set type frame
 		set cl "-class $iclass"
 	} else {
@@ -628,11 +701,11 @@ Widget method init { args } {
 		set args [lrange $args 1 end]
 		set cl ""
 	}
-	if {[info commands $self-cmd] != ""} {
+	if [string compare "" [info commands $self-cmd]] {
 		set win $self
 		set wincmd $self-cmd
 	} else {
-	  if { [string index $self 0] == "." } {
+	  if ![string compare "." [string index $self 0]] {
 		rename $self _ooTmp
 		eval $type $self $cl $args
 		rename $self $self-cmd
@@ -642,11 +715,12 @@ Widget method init { args } {
 	  } else {
 		eval $type .$self $cl $args
 		set win .$self
-		set wincmd .$self-cmd
+		#set wincmd .$self-cmd
+		set wincmd .$self
 	  }
 	}
 	bind $win <Destroy> "\
-		if { \"%W\" == \"$self\" && !\[catch {info args $self}\] } {
+		if { !\[string compare \"%W\" \"$self\"\] && !\[catch {info args $self}\] } {
 			$self destroy -obj_only }"
 
 	return $self
@@ -654,11 +728,11 @@ Widget method init { args } {
 
 # Just for the case when there are no option-handlers defined:
 #
-Widget sys_method configure { args } {
+Widget sys_method configure args {
 	instvar wincmd
 	eval {$wincmd configure} $args
 }
-Widget sys_method cget { opt } {
+Widget sys_method cget opt {
 	instvar wincmd
 	eval {$wincmd cget} $opt
 }
@@ -668,7 +742,7 @@ Widget sys_method configure_unknown { opt args } {
 	eval {$wincmd configure $opt} $args
 }
 
-Widget sys_method cget_unknown { opt } {
+Widget sys_method cget_unknown opt {
 	instvar wincmd
 	$wincmd cget $opt
 }
@@ -683,19 +757,21 @@ Widget sys_method unknown args {
 
 # Note: no "next" used!  Does the `Base::destroy' stuff here for performance.
 #
-Widget method destroy { args } {
+Widget method destroy args {
 	
 	instvar win wincmd
 
-	# Must copy vars since they are destroyed by `freeObj'
+	# Must copy vars since they are destroyed by `otFreeObj'
 	set wp $win
 	set w $wincmd
 
-	freeObj $self
+	otFreeObj $self
 	catch {bind $w <Destroy> {}}
-	if { $args != "-obj_only" } {
-		rename $w $wp
-		if { $args != "-keepwin" } {
+	if [string compare "-obj_only" $args] {
+		if [string compare $w $wp] {
+			rename $w $wp
+		}
+		if [string compare "-keepwin" $args] {
 			destroy $wp
 		}
 	}
@@ -704,12 +780,12 @@ Widget method destroy { args } {
 # The method `set' defined here shadows the `set' method from Base.
 # This allows wrapper objects around Tk-scrollbars to work correctly.
 #
-Widget sys_method set { args } {
+Widget sys_method set args {
 	instvar wincmd
 	eval {$wincmd set} $args
 }
 
-Widget sys_method base_set { args } {
+Widget sys_method base_set args {
 	eval Base::set $args
 }
 
